@@ -22,6 +22,7 @@ import mozilla.telemetry.glean.private.CounterMetricType
 import mozilla.telemetry.glean.private.EventMetricType
 import mozilla.telemetry.glean.private.Lifetime
 import mozilla.telemetry.glean.private.NoExtraKeys
+import mozilla.telemetry.glean.private.NoExtras
 import mozilla.telemetry.glean.private.NoReasonCodes
 import mozilla.telemetry.glean.private.PingType
 import mozilla.telemetry.glean.private.StringMetricType
@@ -84,12 +85,12 @@ class GleanTest {
         // we are only expecting one baseline ping.
         assertEquals(server.requestCount, 2)
 
-        var request = server.takeRequest(20L, TimeUnit.SECONDS)
-        var docType = request.path.split("/")[3]
+        var request = server.takeRequest(20L, TimeUnit.SECONDS)!!
+        var docType = request.path!!.split("/")[3]
         assertEquals("baseline", docType)
 
-        request = server.takeRequest(20L, TimeUnit.SECONDS)
-        docType = request.path.split("/")[3]
+        request = server.takeRequest(20L, TimeUnit.SECONDS)!!
+        docType = request.path!!.split("/")[3]
         assertEquals("metrics", docType)
     }
 
@@ -108,7 +109,7 @@ class GleanTest {
         triggerWorkManager(context)
 
         val request = server.takeRequest(20L, TimeUnit.SECONDS)
-        assertEquals(request.getHeader("X-Debug-ID"), "this-ping-is-tagged")
+        assertEquals(request!!.getHeader("X-Debug-ID"), "this-ping-is-tagged")
     }
 
     // Tests from glean-ac (706af1f).
@@ -174,12 +175,13 @@ class GleanTest {
         assertFalse(Glean.testIsExperimentActive("experiment_preinit_disabled"))
     }
 
+    // Suppressing our own deprecation before we move over to the new event recording API.
     @Test
-    @Suppress("ComplexMethod", "LongMethod")
+    @Suppress("ComplexMethod", "LongMethod", "DEPRECATION")
     fun `test sending of foreground and background pings`() {
         val server = getMockWebServer()
 
-        val click = EventMetricType<NoExtraKeys>(
+        val click = EventMetricType<NoExtraKeys, NoExtras>(
             disabled = false,
             category = "ui",
             lifetime = Lifetime.Ping,
@@ -217,8 +219,9 @@ class GleanTest {
             triggerWorkManager(context)
 
             for (i in 0..5) {
-                val request = server.takeRequest(20L, TimeUnit.SECONDS)
-                val docType = request.path.split("/")[3]
+                val request = server.takeRequest(20L, TimeUnit.SECONDS)!!
+                val docType = request.path!!.split("/")[3]
+
                 val json = JSONObject(request.getPlainBody())
                 checkPingSchema(json)
                 if (docType == "events") {
@@ -274,16 +277,16 @@ class GleanTest {
             // Trigger worker task to upload the pings in the background
             triggerWorkManager(context)
 
-            var request = server.takeRequest(20L, TimeUnit.SECONDS)
-            var docType = request.path.split("/")[3]
+            var request = server.takeRequest(20L, TimeUnit.SECONDS)!!
+            var docType = request.path!!.split("/")[3]
             assertEquals("The received ping must be a 'baseline' ping", "baseline", docType)
 
             val baselineJson = JSONObject(request.getPlainBody())
             assertEquals("dirty_startup", baselineJson.getJSONObject("ping_info")["reason"])
             checkPingSchema(baselineJson)
 
-            request = server.takeRequest(20L, TimeUnit.SECONDS)
-            docType = request.path.split("/")[3]
+            request = server.takeRequest(20L, TimeUnit.SECONDS)!!
+            docType = request.path!!.split("/")[3]
             assertEquals("The received ping must be a 'metrics' ping", "metrics", docType)
         } finally {
             server.shutdown()
@@ -345,7 +348,11 @@ class GleanTest {
     fun `Initializing twice is a no-op`() {
         val beforeConfig = Glean.configuration
 
-        Glean.initialize(context, true)
+        Glean.initialize(
+            context,
+            true,
+            buildInfo = GleanBuildInfo.buildInfo
+        )
 
         val afterConfig = Glean.configuration
 
@@ -470,8 +477,8 @@ class GleanTest {
         triggerWorkManager(context)
 
         // Validate the received data.
-        val request = server.takeRequest(20L, TimeUnit.SECONDS)
-        val docType = request.path.split("/")[3]
+        val request = server.takeRequest(20L, TimeUnit.SECONDS)!!
+        val docType = request.path!!.split("/")[3]
         assertEquals(pingName, docType)
 
         val pingJson = JSONObject(request.getPlainBody())
@@ -568,7 +575,11 @@ class GleanTest {
     @Test(expected = IllegalThreadStateException::class)
     fun `Glean initialize must be called on the main thread`() {
         runBlocking(KotlinDispatchers.IO) {
-            Glean.initialize(context, true)
+            Glean.initialize(
+                context,
+                true,
+                buildInfo = GleanBuildInfo.buildInfo
+            )
         }
     }
 
@@ -592,7 +603,7 @@ class GleanTest {
         // Now trigger execution to ensure the tasks fired
         Glean.initialize(context, true, Glean.configuration.copy(
             serverEndpoint = "http://" + server.hostName + ":" + server.port
-        ))
+        ), GleanBuildInfo.buildInfo)
 
         assertEquals(110, GleanError.preinitTasksOverflow.testGetValue())
 
@@ -603,7 +614,7 @@ class GleanTest {
         // Now trigger it to upload
         triggerWorkManager(context)
 
-        val request = server.takeRequest(20L, TimeUnit.SECONDS)
+        val request = server.takeRequest(20L, TimeUnit.SECONDS)!!
         val jsonContent = JSONObject(request.getPlainBody())
         assertEquals(
             110,
@@ -639,8 +650,8 @@ class GleanTest {
         // Now trigger it to upload
         triggerWorkManager(context)
 
-        val request = server.takeRequest(20L, TimeUnit.SECONDS)
-        val docType = request.path.split("/")[3]
+        val request = server.takeRequest(20L, TimeUnit.SECONDS)!!
+        val docType = request.path!!.split("/")[3]
         assertEquals("deletion-request", docType)
     }
 
@@ -693,8 +704,8 @@ class GleanTest {
             // Trigger worker task to upload the pings in the background
             triggerWorkManager(context)
 
-            val request = server.takeRequest(20L, TimeUnit.SECONDS)
-            val docType = request.path.split("/")[3]
+            val request = server.takeRequest(20L, TimeUnit.SECONDS)!!
+            val docType = request.path!!.split("/")[3]
             assertEquals("The received ping must be a 'baseline' ping", "baseline", docType)
 
             val baselineJson = JSONObject(request.getPlainBody())
@@ -731,14 +742,14 @@ class GleanTest {
         )
 
         Glean.setDebugViewTag("valid-tag")
-        Glean.initialize(context, true, config)
+        Glean.initialize(context, true, config, GleanBuildInfo.buildInfo)
 
         // Send a ping
         Glean.handleBackgroundEvent()
         // Trigger it to upload
         triggerWorkManager(context)
 
-        val request = server.takeRequest(20L, TimeUnit.SECONDS)
+        val request = server.takeRequest(20L, TimeUnit.SECONDS)!!
         assertEquals(request.getHeader("X-Debug-ID"), "valid-tag")
     }
 
@@ -792,7 +803,7 @@ class GleanTest {
         val config = Glean.configuration.copy(
             serverEndpoint = "http://" + server.hostName + ":" + server.port
         )
-        Glean.initialize(context, true, config)
+        Glean.initialize(context, true, config, GleanBuildInfo.buildInfo)
 
         // Glean might still be initializing. Disable upload.
         Glean.setUploadEnabled(false)
@@ -809,8 +820,19 @@ class GleanTest {
         triggerWorkManager(context)
 
         // Validate the received data.
-        val request = server.takeRequest(20L, TimeUnit.SECONDS)
-        val docType = request.path.split("/")[3]
+        val request = server.takeRequest(20L, TimeUnit.SECONDS)!!
+        val docType = request.path!!.split("/")[3]
         assertEquals("deletion-request", docType)
     }
+
+    @Test
+    fun `test passing in explicit BuildInfo`() {
+        Glean.testDestroyGleanHandle()
+        Glean.initialize(
+            context, true, buildInfo = BuildInfo(versionName = "foo", versionCode = "c0ffee")
+        )
+
+        assertEquals("c0ffee", GleanInternalMetrics.appBuild.testGetValue())
+        assertEquals("foo", GleanInternalMetrics.appDisplayVersion.testGetValue())
+    }
 }
