@@ -6,6 +6,7 @@ import Foundation
 
 private typealias GleanBaseline = GleanMetrics.GleanBaseline
 private typealias GleanInternalMetrics = GleanMetrics.GleanInternalMetrics
+private typealias GleanValidation = GleanMetrics.GleanValidation
 private typealias Pings = GleanMetrics.Pings
 
 /// Public exported type identifying individual timers for `TimingDistributionMetricType`
@@ -128,6 +129,14 @@ public class Glean {
                 _ = self.setSourceTags(sourceTags)
             }
 
+            // Get the current value of the dirty flag so we know whether to
+            // send a dirty startup baseline ping below.  Immediately set it to
+            // `false` so that dirty startup pings won't be sent if Glean
+            // initialization does not complete successfully.
+            // It is set to `true` again when registering the lifecycle observer.
+            let isDirtyFlagSet = glean_is_dirty_flag_set().toBool()
+            glean_set_dirty_flag(false.toByte())
+
             // Register builtin pings.
             // Unfortunately we need to manually list them here to guarantee they are registered synchronously
             // before we need them.
@@ -167,13 +176,11 @@ public class Glean {
             // Check if the "dirty flag" is set. That means the product was probably
             // force-closed. If that's the case, submit a 'baseline' ping with the
             // reason "dirty_startup". We only do that from the second run.
-            if !isFirstRun {
-                if glean_is_dirty_flag_set().toBool() {
-                    self.submitPingByNameSync(
-                        pingName: "baseline",
-                        reason: "dirty_startup"
-                    )
-                }
+            if !isFirstRun && isDirtyFlagSet {
+                self.submitPingByNameSync(
+                    pingName: "baseline",
+                    reason: "dirty_startup"
+                )
             }
 
             // From the second time we run, after all startup pings are generated,
@@ -406,6 +413,7 @@ public class Glean {
 
         // Start the timespan for the new activity period.
         GleanBaseline.duration.start()
+        GleanValidation.foregroundCount.add(1)
     }
 
     /// Handle background event and submit appropriate pings
@@ -541,6 +549,16 @@ public class Glean {
         }
     }
 
+    /// Set the source tags to be applied as headers when uploading pings.
+    ///
+    /// If any of the tags is invalid nothing will be set and this function will
+    /// return `false`.
+    /// If Glean is not initialized yet, tags will not be validated at this point.
+    ///
+    /// This is only meant to be used internally by the `GleanDebugActivity`.
+    ///
+    /// - parameters:
+    ///    * tags: A list of tags, which must be valid HTTP header values.
     func setSourceTags(_ value: [String]) -> Bool {
         if self.isInitialized() {
             let len = value.count
