@@ -118,7 +118,10 @@ class GleanTests: XCTestCase {
                 XCTAssertEqual(metrics?.count, 2, "metrics has more keys than expected: \(JSONStringify(metrics!))")
                 let labeledCounters = metrics?["labeled_counter"] as? [String: Any]
                 labeledCounters!.forEach { key, _ in
-                    XCTAssertTrue(key.starts(with: "glean.error"))
+                    XCTAssertTrue(
+                        key.starts(with: "glean.error") || key.starts(with: "glean.validation"),
+                        "Should only see glean.* counters, saw \(key)"
+                    )
                 }
             }
 
@@ -179,7 +182,10 @@ class GleanTests: XCTestCase {
                     XCTAssertEqual(metrics?.count, 2, "metrics has more keys than expected: \(JSONStringify(metrics!))")
                     let labeledCounters = metrics?["labeled_counter"] as? [String: Any]
                     labeledCounters!.forEach { key, _ in
-                        XCTAssertTrue(key.starts(with: "glean.error"))
+                        XCTAssertTrue(
+                            key.starts(with: "glean.error") || key.starts(with: "glean.validation"),
+                            "Should only see glean.* counters, saw \(key)"
+                        )
                     }
                 }
             }
@@ -480,6 +486,21 @@ class GleanTests: XCTestCase {
             XCTAssertNil(error, "Test timed out waiting for upload: \(error!)")
         }
     }
+
+    func testForegroundCounter() {
+        // Glean is started by the test framework.
+        // That already triggers the first foreground event.
+
+        // Put it in the background
+        Glean.shared.handleBackgroundEvent()
+
+        // Bring it back
+        Glean.shared.handleForegroundEvent()
+
+        let foregroundCounter = GleanMetrics.GleanValidation.foregroundCount
+        XCTAssert(foregroundCounter.testHasValue())
+        XCTAssertEqual(2, try foregroundCounter.testGetValue())
+    }
 }
 
 // swiftlint:enable type_body_length
