@@ -3,6 +3,7 @@
 // file, You can obtain one at https://mozilla.org/MPL/2.0/.
 
 use std::env;
+use std::path::PathBuf;
 
 use once_cell::sync::Lazy;
 use tempfile::Builder;
@@ -36,10 +37,10 @@ fn main() {
     let mut args = env::args().skip(1);
 
     let data_path = if let Some(path) = args.next() {
-        path
+        PathBuf::from(path)
     } else {
         let root = Builder::new().prefix("simple-db").tempdir().unwrap();
-        root.path().display().to_string()
+        root.path().to_path_buf()
     };
 
     let cfg = Configuration {
@@ -51,6 +52,7 @@ fn main() {
         channel: None,
         server_endpoint: Some("invalid-test-host".into()),
         uploader: None,
+        use_core_mps: true,
     };
 
     let client_info = ClientInfoMetrics {
@@ -58,8 +60,8 @@ fn main() {
         app_display_version: env!("CARGO_PKG_VERSION").to_string(),
     };
 
-    glean::initialize(cfg, client_info);
     glean::register_ping_type(&PrototypePing);
+    glean::initialize(cfg, client_info);
 
     glean_metrics::sample_boolean.set(true);
 
