@@ -28,6 +28,7 @@
 //!     channel: None,
 //!     server_endpoint: None,
 //!     uploader: None,
+//!     use_core_mps: false,
 //! };
 //! glean::initialize(cfg, ClientInfoMetrics::unknown());
 //!
@@ -151,6 +152,17 @@ where
     f(&mut lock)
 }
 
+/// Launches a new task on the global dispatch queue with a reference to the Glean singleton.
+fn launch_with_glean(callback: impl FnOnce(&Glean) + Send + 'static) {
+    dispatcher::launch(|| crate::with_glean(callback));
+}
+
+/// Launches a new task on the global dispatch queue with a mutable reference to the
+/// Glean singleton.
+fn launch_with_glean_mut(callback: impl FnOnce(&mut Glean) + Send + 'static) {
+    dispatcher::launch(|| crate::with_glean_mut(callback));
+}
+
 /// Creates and initializes a new Glean object.
 ///
 /// See [`glean_core::Glean::new`] for more information.
@@ -161,21 +173,30 @@ where
 /// * `client_info` - the [`ClientInfoMetrics`] values used to set Glean
 ///   core metrics.
 pub fn initialize(cfg: Configuration, client_info: ClientInfoMetrics) {
+    initialize_internal(cfg, client_info);
+}
+
+fn initialize_internal(
+    cfg: Configuration,
+    client_info: ClientInfoMetrics,
+) -> Option<std::thread::JoinHandle<()>> {
     if was_initialize_called() {
         log::error!("Glean should not be initialized multiple times");
-        return;
+        return None;
     }
 
-    std::thread::Builder::new()
+    let init_handle = std::thread::Builder::new()
         .name("glean.init".into())
         .spawn(move || {
             let core_cfg = glean_core::Configuration {
                 upload_enabled: cfg.upload_enabled,
-                data_path: cfg.data_path.clone(),
+                data_path: cfg.data_path,
                 application_id: cfg.application_id.clone(),
                 language_binding_name: LANGUAGE_BINDING_NAME.into(),
                 max_events: cfg.max_events,
                 delay_ping_lifetime_io: cfg.delay_ping_lifetime_io,
+                app_build: client_info.app_build.clone(),
+                use_core_mps: cfg.use_core_mps,
             };
 
             let glean = match Glean::new(core_cfg) {
@@ -275,7 +296,7 @@ pub fn initialize(cfg: Configuration, client_info: ClientInfoMetrics) {
                 // The next times we start, we would have them around already.
                 let is_first_run = glean.is_first_run();
                 if is_first_run {
-                    initialize_core_metrics(&glean, &state.client_info, state.channel.clone());
+                    initialize_core_metrics(glean, &state.client_info, state.channel.clone());
                 }
 
                 // Deal with any pending events so we can start recording new ones
@@ -291,7 +312,7 @@ pub fn initialize(cfg: Configuration, client_info: ClientInfoMetrics) {
                 // Set up information and scheduling for Glean owned pings. Ideally, the "metrics"
                 // ping startup check should be performed before any other ping, since it relies
                 // on being dispatched to the API context before any other metric.
-                // TODO: start the metrics ping scheduler, will happen in bug 1672951.
+                glean.start_metrics_ping_scheduler();
 
                 // Check if the "dirty flag" is set. That means the product was probably
                 // force-closed. If that's the case, submit a 'baseline' ping with the
@@ -302,10 +323,7 @@ pub fn initialize(cfg: Configuration, client_info: ClientInfoMetrics) {
                     // write lock on the `glean` object.
                     // Note that unwrapping below is safe: the function will return an
                     // `Ok` value for a known ping.
-                    if glean
-                        .submit_ping_by_name("baseline", Some("dirty_startup"))
-                        .unwrap()
-                    {
+                    if glean.submit_ping_by_name("baseline", Some("dirty_startup")) {
                         state.upload_manager.trigger_upload();
                     }
                 }
@@ -315,7 +333,7 @@ pub fn initialize(cfg: Configuration, client_info: ClientInfoMetrics) {
                 // Any new value will be sent in newly generated pings after startup.
                 if !is_first_run {
                     glean.clear_application_lifetime_metrics();
-                    initialize_core_metrics(&glean, &state.client_info, state.channel.clone());
+                    initialize_core_metrics(glean, &state.client_info, state.channel.clone());
                 }
             });
 
@@ -324,7 +342,7 @@ pub fn initialize(cfg: Configuration, client_info: ClientInfoMetrics) {
                 Ok(task_count) if task_count > 0 => {
                     with_glean(|glean| {
                         glean_metrics::error::preinit_tasks_overflow
-                            .add_sync(&glean, task_count as i32);
+                            .add_sync(glean, task_count as i32);
                     });
                 }
                 Ok(_) => {}
@@ -336,6 +354,7 @@ pub fn initialize(cfg: Configuration, client_info: ClientInfoMetrics) {
     // Mark the initialization as called: this needs to happen outside of the
     // dispatched block!
     INITIALIZE_CALLED.store(true, Ordering::SeqCst);
+    Some(init_handle)
 }
 
 /// Shuts down Glean.
@@ -352,6 +371,11 @@ pub fn shutdown() {
         return;
     }
 
+    crate::launch_with_glean_mut(|glean| {
+        glean.cancel_metrics_ping_scheduler();
+        glean.set_dirty_flag(false);
+    });
+
     if let Err(e) = dispatcher::shutdown() {
         log::error!("Can't shutdown dispatcher thread: {:?}", e);
     }
@@ -419,7 +443,7 @@ fn initialize_core_metrics(
     if let Some(app_channel) = channel {
         core_metrics::internal_metrics::app_channel.set_sync(glean, app_channel);
     }
-    core_metrics::internal_metrics::os_version.set_sync(glean, "unknown".to_string());
+    core_metrics::internal_metrics::os_version.set_sync(glean, system::get_os_version());
     core_metrics::internal_metrics::architecture.set_sync(glean, system::ARCH.to_string());
     core_metrics::internal_metrics::device_manufacturer.set_sync(glean, "unknown".to_string());
     core_metrics::internal_metrics::device_model.set_sync(glean, "unknown".to_string());
@@ -444,27 +468,24 @@ pub fn set_upload_enabled(enabled: bool) {
     //
     // Because the dispatch queue is halted until Glean is fully initialized
     // we can safely enqueue here and it will execute after initialization.
-    dispatcher::launch(move || {
-        with_glean_mut(|glean| {
-            let state = global_state().lock().unwrap();
-            let old_enabled = glean.is_upload_enabled();
-            glean.set_upload_enabled(enabled);
+    crate::launch_with_glean_mut(move |glean| {
+        let state = global_state().lock().unwrap();
+        let old_enabled = glean.is_upload_enabled();
+        glean.set_upload_enabled(enabled);
 
-            // TODO: Cancel upload and any outstanding metrics ping scheduler
-            // task. Will happen on bug 1672951.
+        if !old_enabled && enabled {
+            glean.start_metrics_ping_scheduler();
+            // If uploading is being re-enabled, we have to restore the
+            // application-lifetime metrics.
+            initialize_core_metrics(glean, &state.client_info, state.channel.clone());
+        }
 
-            if !old_enabled && enabled {
-                // If uploading is being re-enabled, we have to restore the
-                // application-lifetime metrics.
-                initialize_core_metrics(&glean, &state.client_info, state.channel.clone());
-            }
-
-            if old_enabled && !enabled {
-                // If uploading is disabled, we need to send the deletion-request ping:
-                // note that glean-core takes care of generating it.
-                state.upload_manager.trigger_upload();
-            }
-        });
+        if old_enabled && !enabled {
+            glean.cancel_metrics_ping_scheduler();
+            // If uploading is disabled, we need to send the deletion-request ping:
+            // note that glean-core takes care of generating it.
+            state.upload_manager.trigger_upload();
+        }
     });
 }
 
@@ -476,10 +497,8 @@ pub fn register_ping_type(ping: &private::PingType) {
     // Submission itself is also dispatched, so it will always come after the registration.
     if was_initialize_called() {
         let ping = ping.clone();
-        dispatcher::launch(move || {
-            with_glean_mut(|glean| {
-                glean.register_ping_type(&ping.ping_type);
-            })
+        crate::launch_with_glean_mut(move |glean| {
+            glean.register_ping_type(&ping.ping_type);
         })
     } else {
         // We need to keep track of pings, so they get re-registered after a reset or
@@ -542,13 +561,13 @@ pub(crate) fn submit_ping_by_name_sync(ping: &str, reason: Option<&str>) {
             // This won't actually return from `submit_ping_by_name`, but
             // returning `false` here skips spinning up the uploader below,
             // which is basically the same.
-            return Some(false);
+            return false;
         }
 
-        glean.submit_ping_by_name(&ping, reason.as_deref()).ok()
+        glean.submit_ping_by_name(ping, reason.as_deref())
     });
 
-    if let Some(true) = submitted_ping {
+    if submitted_ping {
         let state = global_state().lock().unwrap();
         state.upload_manager.trigger_upload();
     }
@@ -564,14 +583,8 @@ pub fn set_experiment_active(
     branch: String,
     extra: Option<HashMap<String, String>>,
 ) {
-    dispatcher::launch(move || {
-        with_glean(|glean| {
-            glean.set_experiment_active(
-                experiment_id.to_owned(),
-                branch.to_owned(),
-                extra.to_owned(),
-            )
-        });
+    crate::launch_with_glean(move |glean| {
+        glean.set_experiment_active(experiment_id.to_owned(), branch.to_owned(), extra)
     })
 }
 
@@ -579,9 +592,7 @@ pub fn set_experiment_active(
 ///
 /// See [`glean_core::Glean::set_experiment_inactive`].
 pub fn set_experiment_inactive(experiment_id: String) {
-    dispatcher::launch(move || {
-        with_glean(|glean| glean.set_experiment_inactive(experiment_id.to_owned()))
-    })
+    crate::launch_with_glean(move |glean| glean.set_experiment_inactive(experiment_id))
 }
 
 /// Performs the collection/cleanup operations required by becoming active.
@@ -591,16 +602,14 @@ pub fn set_experiment_inactive(experiment_id: String) {
 /// This should be called whenever the consuming product becomes active (e.g.
 /// getting to foreground).
 pub fn handle_client_active() {
-    dispatcher::launch(move || {
-        with_glean_mut(|glean| {
-            glean.handle_client_active();
+    crate::launch_with_glean_mut(|glean| {
+        glean.handle_client_active();
 
-            // The above call may generate pings, so we need to trigger
-            // the uploader. It's fine to trigger it if no ping was generated:
-            // it will bail out.
-            let state = global_state().lock().unwrap();
-            state.upload_manager.trigger_upload();
-        })
+        // The above call may generate pings, so we need to trigger
+        // the uploader. It's fine to trigger it if no ping was generated:
+        // it will bail out.
+        let state = global_state().lock().unwrap();
+        state.upload_manager.trigger_upload();
     });
 
     // The previous block of code may send a ping containing the `duration` metric,
@@ -622,16 +631,14 @@ pub fn handle_client_inactive() {
     // by the next call.
     core_metrics::internal_metrics::baseline_duration.stop();
 
-    dispatcher::launch(move || {
-        with_glean_mut(|glean| {
-            glean.handle_client_inactive();
+    crate::launch_with_glean_mut(|glean| {
+        glean.handle_client_inactive();
 
-            // The above call may generate pings, so we need to trigger
-            // the uploader. It's fine to trigger it if no ping was generated:
-            // it will bail out.
-            let state = global_state().lock().unwrap();
-            state.upload_manager.trigger_upload();
-        })
+        // The above call may generate pings, so we need to trigger
+        // the uploader. It's fine to trigger it if no ping was generated:
+        // it will bail out.
+        let state = global_state().lock().unwrap();
+        state.upload_manager.trigger_upload();
     })
 }
 
@@ -661,6 +668,19 @@ pub(crate) fn test_get_experiment_data(experiment_id: String) -> RecordedExperim
 pub(crate) fn destroy_glean(clear_stores: bool) {
     // Destroy the existing glean instance from glean-core.
     if was_initialize_called() {
+        // Reset the dispatcher first (it might still run tasks against the database)
+        dispatcher::reset_dispatcher();
+
+        // Wait for any background uploader thread to finish.
+        // This needs to be done before the check below,
+        // as the uploader will also try to acquire a lock on the global Glean.
+        //
+        // Note: requires the block here, so we drop the lock again.
+        {
+            let state = global_state().lock().unwrap();
+            state.upload_manager.test_wait_for_upload();
+        }
+
         // We need to check if the Glean object (from glean-core) is
         // initialized, otherwise this will crash on the first test
         // due to bug 1675215 (this check can be removed once that
@@ -675,8 +695,12 @@ pub(crate) fn destroy_glean(clear_stores: bool) {
         }
         // Allow us to go through initialization again.
         INITIALIZE_CALLED.store(false, Ordering::SeqCst);
-        // Reset the dispatcher.
-        dispatcher::reset_dispatcher();
+
+        // If Glean initialization previously didn't finish,
+        // then the global state might not have been reset
+        // and thus needs to be cleared here.
+        let state = global_state().lock().unwrap();
+        state.upload_manager.test_clear_upload_thread();
     }
 }
 
@@ -685,9 +709,9 @@ pub(crate) fn destroy_glean(clear_stores: bool) {
 pub fn test_reset_glean(cfg: Configuration, client_info: ClientInfoMetrics, clear_stores: bool) {
     destroy_glean(clear_stores);
 
-    // Always log pings for tests
-    //Glean.setLogPings(true)
-    initialize(cfg, client_info);
+    if let Some(handle) = initialize_internal(cfg, client_info) {
+        handle.join().unwrap();
+    }
 }
 
 /// Sets a debug view tag.
@@ -738,28 +762,29 @@ pub fn set_log_pings(value: bool) {
 /// Overrides any existing source tags.
 /// Source tags will show in the destination datasets, after ingestion.
 ///
+/// **Note** If one or more tags are invalid, all tags are ignored.
+///
 /// # Arguments
 ///
 /// * `tags` - A vector of at most 5 valid HTTP header values. Individual
 ///   tags must match the regex: "[a-zA-Z0-9-]{1,20}".
-///
-/// # Returns
-///
-/// This will return `false` in case `value` contains invalid tags and `true`
-/// otherwise or if the tag is set before Glean is initialized.
-pub fn set_source_tags(tags: Vec<String>) -> bool {
+pub fn set_source_tags(tags: Vec<String>) {
     if was_initialize_called() {
-        with_glean_mut(|glean| glean.set_source_tags(tags))
+        crate::launch_with_glean_mut(|glean| {
+            glean.set_source_tags(tags);
+        });
     } else {
         // Glean has not been initialized yet. Cache the provided source tags.
         let m = PRE_INIT_SOURCE_TAGS.get_or_init(Default::default);
         let mut lock = m.lock().unwrap();
         *lock = tags;
-        // When setting the source tags before initialization,
-        // we don't validate the tags, thus this function always returns true.
-        true
     }
 }
 
+/// Returns a timestamp corresponding to "now" with millisecond precision.
+pub fn get_timestamp_ms() -> u64 {
+    glean_core::get_timestamp_ms()
+}
+
 #[cfg(test)]
 mod test;
